function A = intersection_area(rect1, rect2, do_vis)
  if ~exist('do_vis', 'var')
    do_vis = false;
  end

  A = 0;

  % special case is rectangles are equal
  if sum((rect1(:) - rect2(:)).^2) < eps
    A = rect_area(rect1);
    return;
  end

  points = get_intersection_points(rect1, rect2);

  if isempty(points)
    if do_vis
      fprintf('no intersection\n');
    end

    return;
  end

  % todo: eliminate duplicate points

  % find the lowest point, if several choose one with smallest x (assume no duplicate points)
  li = 1;
  npoints = size(points,1);

  for i = 2:npoints
    if points(i, 2) < points(li, 2)
      li = i;
    elseif abs(points(i, 2) - points(li, 2)) < eps && points(i, 1) < points(li, 1)
      li = i;
    end
  end

  % compute tangent for each point
  postan = [];
  negtan = [];

  for i = 1:npoints
    if i ~= li
      d = points(i, :) - points(li, :);

      %assert(d(2) > 0);
      %assert(d(1) ~= 0);

      if d(1) == 0
        ptan = Inf;
      else
        ptan = d(2) / d(1);
      end

      if d(1) > 0
        postan = [postan; ptan, i];
      else
        negtan = [negtan; ptan, i];
      end

    end   
  end

  if ~isempty(postan)
    postan;
    postan = sortrows(postan, 1);
  end

  if ~isempty(negtan)
    negtan;
    negtan = sortrows(negtan, 1);
  end

  pointorder = [postan; negtan];

  if do_vis
    plot(points(:, 1), points(:, 2), 'ro');
    plot(points(li, 1), points(li, 2), 'go');

    for i = 1:npoints-1
      ptidx = pointorder(i, 2);
      plot([points(li, 1), points(ptidx, 1)], [points(li, 2), points(ptidx, 2)], '-b');
    end
  end

  assert(size(pointorder, 1) == npoints - 1);

  ptidx1 = li;

  for i = 1:npoints - 2
    ptidx2 = pointorder(i, 2);
    ptidx3 = pointorder(i+1, 2);
    CA = tri_area(points(ptidx1, :), points(ptidx2, :), points(ptidx3, :));

    if do_vis
      fprintf('triangle %d, area: %f\n', i, CA);
    end

    A = A + CA;
  end

  if do_vis
    axis equal;
    axis ij;

    fprintf('intersection area: %f\n', A);
  end
end

% return edge intersections and corners of one rect which are inside the other rect
function points = get_intersection_points(rect1, rect2)
  points = [];

  % test edges
  for i1 = 1:4
    for i2 = 1:4
      
      idx1 = mod(i1, 4) + 1;
      idx2 = mod(i2, 4) + 1;
      
      p = segment_intersect(rect1(i1, :), rect1(idx1, :), ...
                            rect2(i2, :), rect2(idx2, :));
      if ~isempty(p)
        points = [points; p];
      end
    end
  end


  points_inside1 = points_in_rect2(rect1, rect2);
  points = [points; points_inside1];
  points_inside2 = points_in_rect2(rect2, rect1);
  points = [points; points_inside2];
  
  points
end

function points_inside = points_in_rect2(points, rect)
  points_inside = [];

  npoints = size(points, 1);

  for ptidx = 1:npoints
    inside = true;

    x = points(ptidx, 1);
    y = points(ptidx, 2);

    sign_dir = 0;

    for i = 1:4
      ni = mod(i, 4) + 1;

      x1 = rect(i, 1);
      y1 = rect(i, 2);
      x2 = rect(ni, 1);
      y2 = rect(ni, 2);

      %s = (x - x1)*(x2 - x1) + (y - y1)*(y2 - y1);
      s = -(x - x1)*(y2 - y1) + (y - y1)*(x2 - x1);

      % BEGIN DEBUG
      %if x == 163 && y == 142
      %  fprintf('x1: %f, y1: %f, x2: %f, y2: %f, s: %f\n', x1, y1, x2, y2, s);
      %end
      % END DEBUG


      if i == 1
        dir_sign = sign(s);
      else
        if sign(s) ~= dir_sign
          inside = false;
          break;
        end
      end

%      if s < 0
%        inside = false;
%        break;
%      end

    end

    if inside 
      points_inside = [points_inside; points(ptidx, :)];
    end
  end


end

% segment1: p1, p2
% segment2: p3, p4
function p = segment_intersect(p1, p2, p3, p4)
  x1 = p1(1);
  y1 = p1(2);

  x2 = p2(1);
  y2 = p2(2);

  x3 = p3(1);
  y3 = p3(2);

  x4 = p4(1);
  y4 = p4(2);

  disp([x1, y1, x2, y2, x3, y3, x4, y4]);

  d = (y4 - y3)*(x2 - x1) - (x4 - x3)*(y2 - y1);

  p = [];  

  % test if segments are parallel
  if (d ~= 0)
    n1 = (x4 - x3)*(y1 - y3) - (y4 - y3)*(x1 - x3);
    n2 = (x2 - x1)*(y1 - y3) - (y2 - y1)*(x1 - x3);
    ua = n1/d;
    ub = n2/d;
    disp([ua, ub, d]);

    %fprintf('segment_intersect: %f, %f\n', ua, ub);

    % test if intersection is whithin both segments (including segment boundaries)
    %if (ua > 0 && ua < 1 && ub > 0 && ub < 1) 
    if (ua >= 0 && ua <= 1 && ub >= 0 && ub <= 1) 
      p = [x1 + ua*(x2 - x1), y1 + ua*(y2 - y1)];
    end
  end

end
