%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2D Annotation Code for:
%
% Are Cars Just 3D Boxes? - Jointly Estimating the 3D Shape of Multiple Objects
% M Zeeshan Zia, Michael Stark, Konrad Schindler
% IEEE Conference on Computer Vision and Pattern Recognition (CVPR), 2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all;
close all;
clc;
load Zia_CVPR14_imagelist.mat;

init_imageid = 1;
final_imageid = 260;

padding = uint8(zeros(375,100,3));
maxwinsize = [1900,60,1600,1000];

KITTI_testimage = [];

for imno = init_imageid:final_imageid
    str = sprintf('../../%06d.png',images_to_include(imno));
    im = imread(str);
    [imr,imc,clrsimg] = size(im);
    padding = uint8(zeros(imr,100,3));

    im = [im,padding];
    objects = readLabels('../../training/label_2',images_to_include(imno));
    
    [a,b] = size(objects);
    KITTI_testimage(imno).imageid = images_to_include(imno);
    cntobj = 0;
    for objectid = 1:b
        if (objects(objectid).y1 == 0)
            objects(objectid).y1=1;
        end
%        if (sum(objects(objectid).type(1:3) == 'Car')==3 || sum(objects(objectid).type(1:3) == 'Van')==3 )
        if (sum(objects(objectid).type(1:3) == 'Car')==3)
            P_mat = readCalibration('../../training/calib',images_to_include(imno),2);
            [corners,face_idx] = computeBox3D(objects(objectid),P_mat); % compute 2D bbx, as projection of 3D bbx
            [acorners,bcorners] = size(corners);
            
            if ((objects(objectid).y2 - objects(objectid).y1) >= 40 && (acorners+bcorners) > 0) % only label cars that are larger than 40 pixels in height (as in KITTI paper, Sec 3.4)
                cntobj = cntobj + 1;

                clc;
                fprintf('Image ID = %d\n',imno);
                fprintf('objectid = %d\n',objectid);

                KITTI_testimage(imno).car(cntobj).x1 = min(corners(1,:));  % maybe negative/ outside image boundaries, specially for the truncated objects
                KITTI_testimage(imno).car(cntobj).x2 = max(corners(1,:));
                KITTI_testimage(imno).car(cntobj).y1 = min(corners(2,:));
                KITTI_testimage(imno).car(cntobj).y2 = max(corners(2,:));

                pt_topleft = [objects(objectid).x1,objects(objectid).y1];
                imcut = im(objects(objectid).y1:objects(objectid).y2,objects(objectid).x1+1:objects(objectid).x2+49,:);
                imcut(1:(objects(objectid).y2-objects(objectid).y1),1:(objects(objectid).x2-objects(objectid).x1),1) =imcut(1:(objects(objectid).y2-objects(objectid).y1),1:(objects(objectid).x2-objects(objectid).x1),1)+50;
                imshow(imcut);
%                set(gcf,'position',maxwinsize);            
                set(gcf,'units','normalized','outerposition',[0 0 0.5 1])

                KITTI_testimage(imno).car(cntobj).objectid = objectid;
                KITTI_testimage(imno).car(cntobj).type = objects(objectid).type;
                angl = (objects(objectid).alpha + pi) * 180/ pi;
                KITTI_testimage(imno).car(cntobj).VPanglediscrete = 45 *(closest_viewpoint(angl,8) - 1);
                fprintf('VP = %d\n',45 *(closest_viewpoint(angl,8) - 1));
                KITTI_testimage(imno).car(cntobj).VPangle = angl;
                partslist = part_from_viewpoint(angl);

                KITTI_testimage(imno).car(cntobj).partid=[];
                KITTI_testimage(imno).car(cntobj).part_x = [];
                KITTI_testimage(imno).car(cntobj).part_y = [];
                KITTI_testimage(imno).car(cntobj).occluded = [];
                for partno = 1:length(partslist)
                    partslist(partno)
                    ptannotated = ginput(1);
                    KITTI_testimage(imno).car(cntobj).partid = [KITTI_testimage(imno).car(cntobj).partid,partslist(partno)];
                    if (ptannotated(1) > (objects(objectid).x2-objects(objectid).x1)+2)
                        KITTI_testimage(imno).car(cntobj).part_x = [KITTI_testimage(imno).car(cntobj).part_x,-1];
                        KITTI_testimage(imno).car(cntobj).part_y = [KITTI_testimage(imno).car(cntobj).part_y,-1]; 
                        KITTI_testimage(imno).car(cntobj).occluded = [KITTI_testimage(imno).car(cntobj).occluded,1]; 
                    else
                        KITTI_testimage(imno).car(cntobj).part_x = [KITTI_testimage(imno).car(cntobj).part_x,ptannotated(1) + pt_topleft(1)];
                        KITTI_testimage(imno).car(cntobj).part_y = [KITTI_testimage(imno).car(cntobj).part_y,ptannotated(2) + pt_topleft(2)];
                        KITTI_testimage(imno).car(cntobj).occluded = [KITTI_testimage(imno).car(cntobj).occluded,0]; 
                    end
                end
            end
        end
    end
    save KITTI_CVPR14_labelled_260ims.mat KITTI_testimage;
end